
#include <cmath>
#include <complex>
#include <algorithm>
#include <iostream>

#include "hark-lib/node/SpectralGainFilter.h"

namespace hark
{
namespace node
{
    SpectralGainFilter::SpectralGainFilter():
        NodeDef("SpectralGainFilter")
    {
        *this
            << Terminal::in("INPUT_SPEC")
            << Terminal::in("VOICE_PROB")
            << Terminal::in("GAIN")
            >> Terminal::out("OUTPUT_SPEC");
    }

    void
    SpectralGainFilter::reset()
    {
    }

    void
    SpectralGainFilter::initialize(InputDataBundle &input)
    {
    }

    bool
    SpectralGainFilter::operator () (size_t count, InputDataBundle &input, OutputDataBundle &output)
    {
        //std::vector<std::complex<float> >
        //std::vector<float>
        //std::vector<float>
        const auto &input_spectrum = input["INPUT_SPEC"].last().cast<std::map<int, data> >();
        const auto &input_voice_probability = input["VOICE_PROB"].last().cast<std::map<int, data> >();
        const auto &input_optimum_gain = input["GAIN"].last().cast<std::map<int, data> >();

#ifdef DEBUG
        ptime start_time;
        ptime end_time;
        start_time = microsec_clock::local_time();
#endif

        //if(count == 0){ this->initialize(input); }

        //std::vector<std::complex<float> >
        //std::vector<std::complex<float> >
        auto out = data::create<std::map<int, data> >();
        auto &output_spectrum = out.cast<std::map<int, data> >();

        if(input_spectrum.size() > 0){
            //std::vector<std::complex<float> >
            std::map<int, data>::const_iterator it_in_spec = input_spectrum.begin();

            for(; it_in_spec != input_spectrum.end(); ++it_in_spec){
                std::map<int, data>::const_iterator it_vp = input_voice_probability.find(it_in_spec->first);
                std::map<int, data>::const_iterator it_gain = input_optimum_gain.find(it_in_spec->first);

                //if(input_voice_probability.find(it_in_spec->first) == input_voice_probability.end()){
                if(it_vp == input_voice_probability.end()){
                    std::stringstream ss;
                    ss << "SpectralGainFilter: Failed to find ID in VOICE_PROB.\n" << "FILE: " << __FILE__ << ", LINE: " << __LINE__;
                    throw RuntimeError(ss.str());
                }
                //if(input_optimum_gain.find(it_in_spec->first) == input_optimum_gain.end()){
                if(it_gain == input_optimum_gain.end()){
                    std::stringstream ss;
                    ss << "SpectralGainFilter: Failed to find ID in GAIN.\n" << "FILE: " << __FILE__ << ", LINE: " << __LINE__;
                    throw RuntimeError(ss.str());
                }
                const std::vector<std::complex<float> > &input_spec = it_in_spec->second.cast<std::vector<std::complex<float> > >();
                const std::vector<float> &vp_vector = it_vp->second.cast<std::vector<float> >();
                const std::vector<float> &gain_vector = it_gain->second.cast<std::vector<float> >();

                std::vector<std::complex<float> > output_vector;
                output_vector.resize(input_spec.size());

                for(size_t k = 0; k < input_spec.size(); k++){
                    if(k < vp_vector.size()){
                        output_vector[k] = input_spec[k] * vp_vector[k] * gain_vector[k];
                    }
                    else{
                        output_vector[k] = input_spec[k];
                    }
                    output_spectrum.emplace(it_in_spec->first, data(output_vector));
                }
            }
        }

        output["OUTPUT_SPEC"].push(out);

#ifdef DEBUG
        end_time = microsec_clock::local_time();
        std::cout << end_time - start_time << std::endl;
#endif

        return true;
    }
}
}

/* end of file */
